@@ -1,24 +1,24 @@
 # Copyright Avraham Adler (c) 2023
 # SPDX-License-Identifier: MPL-2.0+
 
-# Printing convenience function
+# Printing convenience function.
 fC <- function(x, d = 6L, f = "g", w = -1L) {
   formatC(x, digits = d, format = f, width = w)
 }
 
-# Default Chebyshev nodes
+# Default Chebyshev nodes.
 chebNodes <- function(n, a, b) {
   n <- as.integer(n)
   sort(0.5 * (a + b + (b - a) * cos((2 * seq_len(n) - 1) * pi / (2 * n))))
 }
 
-# Create Vandermonde matrix to a polynomial degree n, or n + 1 terms
+# Create Vandermonde matrix to a polynomial degree n, or n + 1 terms.
 vanderMat <- function(x, n) {
-  n1 <- n + 1L
-  matrix(rep(x, each = n1) ^ (seq_len(n1) - 1L), ncol = n1, byrow = TRUE)
+  np1 <- n + 1L
+  matrix(rep(x, each = np1) ^ (seq_len(np1) - 1L), ncol = np1, byrow = TRUE)
 }
 
-# Call the function being approximated on the points x
+# Call the function being approximated on the points x.
 callFun <- function(fn, x) {
   if (!is.function(fn)) stop("Unable to parse function.")
   do.call(match.fun(fn), args = list(x = x))
@@ -27,23 +27,14 @@ callFun <- function(fn, x) {
 # Check that the values passed are oscillating in sign
 isOscil <- function(x) all(abs(diff(sign(x))) == 2)
 
-# Calculate the polynomial approximation. Use in numer & denom for rationals
-# Use Horner's method. This version is fastest of few tried (recursion, Reduce,
-# etc.)
-# polyCalc <- function(x, a) {
-#   ret <- double(length(x))
-#   # Using fastest sequence constructor despite it not checking for empty vector
-#   # as that should not be possible.
-#   for (i in length(a):1L) {
-#     ret <- (ret * x) + a[i]
-#   }
-#   ret
-# }
+# polyCalc uses a Compensated Horner Method based on Langlois et al.(2006)
+# https://drops.dagstuhl.de/opus/volltexte/2006/442/
+# As primary bottleneck, it was ported to C for speed.
+polyCalc <- function(x, a) {
+  .Call(compHorner_c, as.double(x), as.double(a))
+}
 
-# Using Compensated Horner Scheme of Langlois et al. (2006)
-polyCalc <- function(x, a) compensatedHorner(x, a)
-
-# Function to calculate value of minimax approximation at x given a & b
+# Function to calculate value of minimax approximation at x given a & b.
 evalFunc <- function(x, R) {
   ret <- polyCalc(x, R$a)
   if ("b" %in% names(R)) {
@@ -52,7 +43,7 @@ evalFunc <- function(x, R) {
   ret
 }
 
-# Function to calculate error between known and calculated values
+# Function to calculate error between known and calculated values.
 remErr <- function(x, R, fn, relErr) {
   if (relErr) {
     y <- callFun(fn, x)
@@ -63,16 +54,16 @@ remErr <- function(x, R, fn, relErr) {
 }
 
 # Function to identify roots of the error equation for use as bounds in finding
-# the maxima and minima
+# the maxima and minima.
 findRoots <- function(x, R, fn, relErr) {
   r <- double(length(x) - 1L)
   for (i in seq_along(r)) {
     intv <- c(x[i], x[i + 1L])
-    root <- tryCatch(uniroot(remErr, interval = intv, R = R, fn = fn,
-                             relErr = relErr),
+    root <- tryCatch(uniroot(remErr, interval = intv, extendInt = "no", R = R,
+                             fn = fn, relErr = relErr),
                      error = function(cond) simpleError(trimws(cond$message)))
 
-    # If there is no root in the interval, take the endpoint closest to zero
+    # If there is no root in the interval, take the endpoint closest to zero.
     if (inherits(root, "simpleError")) {
       r[i] <- intv[which.min(abs(intv))]
     } else {
@@ -121,34 +112,34 @@ switchX <- function(r, l, u, R, fn, relErr) {
     # Test for 0 value at function if relative error
     if (relErr && callFun(fn, x[i]) == 0) {
       stop("Algorithm is choosing basis point where functional value is ",
-           "0. Please approximate using absolute, and not relative, error.")
+           "0. Please approximate using absolute---not relative---error.")
     }
 
-    # Flip maximize
+    # Flip maximize.
     maximize <- !maximize
   }
   x
 }
 
-# Check Remez iterations for convergence
+# Check Remez iterations for convergence.
 isConverged <- function(errs, expe, convrat, tol) {
   aerrs <- abs(errs)
   mxae <- max(aerrs)
   mnae <- min(aerrs)
 
-  # Check observed errors are close enough to expected by ratio or tolerance
+  # Check observed errors are close enough to expected by ratio or tolerance.
   errDistance <- mxae / expe <= convrat || abs(mxae - expe) <= tol
 
-  # Check observed errors are close enough to each other by ratio or tolerance
+  # Check observed errors are close enough to each other by ratio or tolerance.
   errMagnitude <- mxae / mnae <= convrat || mxae - mnae <= tol
 
-  # Converged if magnitude and distance are close and error oscillates in sign
+  # Converged if magnitude and distance are close and error oscillates in sign.
   isOscil(errs) && errDistance && errMagnitude
 }
 
-# Check denominator polynomial for zero in the requested range
+# Check denominator polynomial for zero in the requested range.
 checkDenom <- function(a, l, u) {
-  dngrRt <- tryCatch(uniroot(polyCalc, c(l, u), a = a),
+  dngrRt <- tryCatch(uniroot(polyCalc, c(l, u), extendInt = "no", a = a),
                      error = function(cond) simpleError(trimws(cond$message)))
   if (inherits(dngrRt, "simpleError")) {
     return(NULL)
@@ -156,3 +147,14 @@ checkDenom <- function(a, l, u) {
     return(dngrRt$root)
   }
 }
+
+# Check for coefficient irrelevancy.
+checkIrrelevant <- function(a, l, u, zt) {
+  if (!is.null(zt) && length(a) > 0) {
+    xmax <- max(abs(l), abs(u))
+    for (i in seq_along(a)) {
+      if (abs(a[i] * xmax ^ (i - 1L)) <= zt) a[i] <- 0
+    }
+  }
+  a
+}
